#include "include/common.h"

int http_proc( struct thread_t *self )
{
	extern struct config_t	config;
	char			uri[1024*4];
	char			*last_modify_char;
	int			last_modify_char_len;
	char			date_822[48];
	int			response_res;

	if ( self->request.header.db_name[self->request.header.db_name_len-1] == '/' )
	{
		snprintf( uri, 1024*4, "%s/%s%s", config.http_root, self->request.header.db_name, HTTP_DEFAULT_INDEX );
	}
	else
	{
		snprintf( uri, 1024*4, "%s/%s", config.http_root, self->request.header.db_name );
	}

	if ( lstat( uri, &self->response.stat ) == SQLITE_OK )
	{
		last_modify_char_len = request_get_char( &last_modify_char, &self->request, "IF-MODIFIED-SINCE" );
		if ( last_modify_char_len > 0 )
		{
			http_time2asctime( date_822, self->response.stat.st_mtime );
			if ( strncmp( last_modify_char, date_822, last_modify_char_len ) == 0 )
			{
				self->response.status = 304;
				self->response.status_msg = "Not Modified";
				self->response.offset = -1;
				self->response.file_offset = 0;
				return response( self );
			}
		}

		self->response.file = open( uri, O_RDONLY );
		if ( self->response.file < 0 )
		{
			return response_error( self, RESPONSE_ERR_INTERNAL, NULL );
		}

		self->response.status = 200;
		self->response.status_msg = "OK";
		self->response.offset = -1;
		self->response.file_offset = 0;
		response_res = response( self );
		close( self->response.file );
		self->response.file = -1;
		return response_res;
	}
	else
	{
		return response_error( self, RESPONSE_ERR_NOT_FOUND, NULL );
	}

	return TRUE;
}

time_t http_asctime2time_t( char *asctime )
{
	int             asc_len;
	struct tm       time;
	short           flag = 0;

	asc_len = strlen( asctime );

	time.tm_wday = time.tm_yday = time.tm_isdst = 0;

	if ( asc_len <= 24 )
	{
		/*Sun Nov 6 08:49:30 1994         ; ANSI C's asctime() format*/
		time.tm_year =  atoi( &(asctime[( asc_len - 4 )]) ) - 1900;
		asctime[( asc_len - 5 )] =      '\0';
		time.tm_sec =   atoi( &(asctime[( asc_len - 7 )]) );
		time.tm_min =   atoi( &(asctime[( asc_len - 10 )]) );
		time.tm_hour =  atoi( &(asctime[( asc_len - 13 )]) );
		asctime[( asc_len - 14 )] =     '\0';
		asctime[7] =    '\0';
		time.tm_mon =   http_ascmon2int( &(asctime[4]) );
		time.tm_mday =  atoi( &(asctime[8]) );
		flag = 1;
	}
	else if ( asc_len == 29 )
	{
		/*Sun, 06 Nov 1994 08:49:30 GMT   ; RFC 822, updated by RFC 1123*/
		if ( memcmp( &(asctime[( asc_len - 3 )]), "GMT", 3 ) == 0 )
		{
			time.tm_mday =  atoi( &(asctime[5]) );
			time.tm_mon =   http_ascmon2int( &(asctime[8]) );
			asctime[16] = '\0';
			time.tm_year =  atoi( &(asctime[12]) ) - 1900;
			time.tm_hour =  atoi( &(asctime[17]) );
			time.tm_min =   atoi( &(asctime[20]) );
			time.tm_sec =   atoi( &(asctime[23]) );
			flag =1;
		}
	}
	else if ( asc_len < 36 )
	{
		/*Sunday, 06-Nov-94 08:49:30 GMT  ; RFC 850, obsoleted by RFC 1036*/
		if ( memcmp( &(asctime[( asc_len - 3 )]), "GMT", 3 ) == 0 )
		{
			time.tm_sec =   atoi( &(asctime[( asc_len - 6 )]) );
			time.tm_min =   atoi( &(asctime[( asc_len - 9 )]) );
			time.tm_hour =  atoi( &(asctime[( asc_len - 12 )]) );
			asctime[( asc_len - 13 )] = '\0';
			time.tm_year =  atoi( &(asctime[( asc_len - 15 )]) );
			time.tm_mon =   http_ascmon2int( &(asctime[( asc_len - 19 )]) );
			time.tm_mday =  atoi( &(asctime[( asc_len - 22 )]) );
			flag =1;
		}
	}

	return mktime(&time);
}

int http_ascmon2int( char *ascmonth )
{
	if ( memcmp( ascmonth, "Jan", 3 ) == 0 )       return 0;
	else if ( memcmp( ascmonth, "Feb", 3 ) == 0 )  return 1;
	else if ( memcmp( ascmonth, "Mar", 3 ) == 0 )  return 2;
	else if ( memcmp( ascmonth, "Apr", 3 ) == 0 )  return 3;
	else if ( memcmp( ascmonth, "May", 3 ) == 0 )  return 4;
	else if ( memcmp( ascmonth, "Jun", 3 ) == 0 )  return 5;
	else if ( memcmp( ascmonth, "Jul", 3 ) == 0 )  return 6;
	else if ( memcmp( ascmonth, "Aug", 3 ) == 0 )  return 7;
	else if ( memcmp( ascmonth, "Sep", 3 ) == 0 )  return 8;
	else if ( memcmp( ascmonth, "Oct", 3 ) == 0 )  return 9;
	else if ( memcmp( ascmonth, "Nov", 3 ) == 0 )  return 10;
	else if ( memcmp( ascmonth, "Dec", 3 ) == 0 )  return 11;
	return -1;
}

int http_time2asctime( char *dst, time_t time )
{
	struct tm		time_s;

	if ( dst == NULL )
		return -1;

	gmtime_r( &time, &time_s );
	return strftime( dst, 48, "%a, %d %b %G %H:%M:%S GMT", &time_s );

	dst[0] = '\0';

	return 0;
}
