#include "include/common.h"

int request_header_parse( struct request_t *request )
{
	extern regex_t	reg_protocol;
	extern regex_t	reg_method;
	extern regex_t	reg_request;
	regmatch_t	method[3];
	size_t		method_cnt = 3;
	int		method_len = 0;
	regmatch_t	header[4];
	size_t		header_cnt = 4;
	int		header_match_idx;
	int		header_len;
	int		match_res;

	// (\\\n\\\n)|(\\\r\\\n\\\r\\\n)|(\\\r\\\r)
	match_res = regexec( &reg_protocol, request->buf, method_cnt, method, 0 );
	header_len = method[0].rm_eo;

	request->header.node_idx=0;

	if ( match_res == 0 )
	{
		// ^([A-Z]+) \\/([^ ]*) HTTP\\/1\\.1
		match_res = regexec( &reg_method, request->buf, method_cnt, method, 0 );
		if ( match_res == 0 )
		{
			method_len = method[0].rm_eo;
			header_match_idx = method_len;

			if ( strncasecmp( &(request->buf[method[1].rm_so]), "POST", 4 ) == 0 )
				request->header.method = REQUEST_METHOD_POST;
			else if ( strncasecmp( &(request->buf[method[1].rm_so]), "GET", 3 ) == 0 )
				request->header.method = REQUEST_METHOD_GET;
			else if ( strncasecmp( &(request->buf[method[1].rm_so]), "HEAD", 4 ) == 0 )
				request->header.method = REQUEST_METHOD_HEAD;

			request->header.db_name = &(request->buf[method[2].rm_so]);
			request->header.db_name_len = method[2].rm_eo-method[2].rm_so;
			request->header.db_name[request->header.db_name_len] = '\0';
			//printf("DB: (%s)\n", request->header.db_name);

			request->header.query = strchr( request->header.db_name, '?' );
			if ( request->header.query != NULL )
			{
				request->header.query[0] = '\0';
				request->header.query++;
			}

			header_match_idx++;
			while ( header_match_idx < header_len )
			{
				// ([a-zA-z-]+)\\s?:\\s?([^\\\n\\\r]+)
				match_res = regexec( &reg_request, &(request->buf[header_match_idx]), header_cnt, header, 0 );
				if ( match_res == 0 )
				{
					request->header.node[request->header.node_idx].field = &(request->buf[header_match_idx + header[1].rm_so]);
					request->header.node[request->header.node_idx].field_len = header[1].rm_eo - header[1].rm_so;
					request->header.node[request->header.node_idx].field[request->header.node[request->header.node_idx].field_len] = '\0';

					request->header.node[request->header.node_idx].value = &(request->buf[header_match_idx + header[2].rm_so]);
					request->header.node[request->header.node_idx].value_len = header[2].rm_eo - header[2].rm_so;
					request->header.node[request->header.node_idx].value[request->header.node[request->header.node_idx].value_len] = '\0';

					//printf("(%s): (%s)\n", request->header.node[request->header.node_idx].field, request->header.node[request->header.node_idx].value);

					header_match_idx += header[0].rm_eo+1;
					request->header.node_idx++;
				}
				else
					break;
			}
		}
		request->header_len = header_len;
	}

	return 0;
}

int request_end( struct request_t *request )
{
	int	content_length;

	content_length = request_get_int( request, "CONTENT-LENGTH" );
	if ( content_length >= 0 )
	{
		if ( request->read_len >= content_length + request->header_len )
		{
			request->data = &(request->buf[request->header_len]);
			if ( memcmp( request->data, "query=", 6) == 0 )
			{
				request->data += 6;
				content_length -= 6;
			}

			if ( request->header_len > 0 )
				content_length = request_decode( request->data, request->data, content_length );
			//printf("%s\n", request->data);
			request->data_len = content_length;
			return SQLITE_OK;
		}
		return SQLITE_EMPTY;
	}
	return SQLITE_ERROR;
}

int request_decode( char *to, char *from, int from_len )
{
	int	len;

	for ( len=0; from_len--; ++to, ++from, ++len )
	{
		if ( from[0] == '%' && from[1] == '%')
		{
			--to;
			--len;
			continue;
		}

		if ( from[0] == '%' && isxdigit( from[1] ) && isxdigit( from[2] ) )
		{
			*to = request_decode_hexit( from[1] ) * 16 + request_decode_hexit( from[2] );
			from += 2;
			from_len -= 2;
		}
		else if ( from[0] == '+' )
			*to = ' ';
		else
			*to = *from;
	}
	*to = '\0';
	return len;
}

int request_decode_hexit( char c )
{
	if ( c >= '0' && c <= '9' )
		return c - '0';
	if ( c >= 'a' && c <= 'f' )
		return c - 'a' + 10;
	if ( c >= 'A' && c <= 'F' )
		return c - 'A' + 10;
	return 0;
}

short request_keep_alive( struct request_t *request )
{
	char	*connection;
	int	connection_len;

	connection_len = request_get_char( &connection, request, "CONNECTION" );
	if ( connection_len > 0 )
		return ( strncasecmp( connection, "KEEP-ALIVE", connection_len ) == 0 )?TRUE:FALSE;

	return FALSE;
}

int request_get_char( char **dst, struct request_t *request, char *field )
{
	int	i;

	if ( request == NULL || field == NULL )
		return -1;

	for ( i=request->header.node_idx; i--; )
	{
		if ( strcasecmp( request->header.node[i].field, field ) == 0 )
		{
			(*dst) = request->header.node[i].value;
			return request->header.node[i].value_len;
		}
	}
	return -1;
}

int request_get_int( struct request_t *request, char *field )
{
	int	i;

	if ( request == NULL || field == NULL )
		return -1;

	for ( i=request->header.node_idx; i--; )
	{
		if ( strcasecmp( request->header.node[i].field, field ) == 0 )
			return atoi(request->header.node[i].value);
	}
	return -1;
}

#define decode(c) if(c >= 'A' && c <= 'Z') c  = c - 'A'; \
		else if(c >= 'a' && c <= 'z') c  = c - 'a' + 26; \
		else if(c >= '0' && c <= '9') c  = c - '0' + 52; \
		else if(c == '+')             c  = 62; \
		else if(c == '/')             c  = 63; \
		else                          c  = 0; \

int base64_decode( char *dst, const char *src, unsigned int len )
{
	int x, y = 0;
	char triple[3];
	char quad[4];

	len = strlen(src);

	for( x = 0; x < len; x += 4 )
	{
		memset(quad, 0, 4);
		memcpy(quad, &src[x], 4 - (len - x) % 4);

		decode(quad[0]);
		decode(quad[1]);
		decode(quad[2]);
		decode(quad[3]);

		triple[0] = (quad[0] << 2) | quad[1] >> 4;
		triple[1] = ((quad[1] << 4) & 0xF0) | quad[2] >> 2;
		triple[2] = ((quad[2] << 6) & 0xC0) | quad[3];

		memcpy(&dst[y], triple, 3);            
		y += 3;      
	}
	return y;
}
